local ns = 'UX'

local DELAY_SUBS_LOADED = false -- Support for DelaySubs addon - eventHandler

local UX_Dialogue = {}

UX_Dialogue.savedVarsVersion = 1
UX_Dialogue.defaults = {
	showIcons = true,
	showNumbers = true,
	showAudioRepeat = true,
	highlightSpecial = true
}

local wm = WINDOW_MANAGER
local em = EVENT_MANAGER

local strformat = string.format

local ZO_OptionsWindowSettingsScrollChild = ZO_OptionsWindowSettingsScrollChild
local ZO_OptionsWindow_InitializeControl = ZO_OptionsWindow_InitializeControl

local ZO_InteractWindowTargetAreaBodyText = ZO_InteractWindowTargetAreaBodyText
local ZO_InteractWindowRewardArea = ZO_InteractWindowRewardArea

local MOUSE_BUTTON_LEFT = 1
--local MOUSE_BUTTON_RIGHT = 2
--local MOUSE_BUTTON_MIDDLE = 3

local CHATTER_ACCEPT_QUEST_BESTOWAL_ALT = 42
local CHATTER_COMPLETE_QUEST_CONFIRM_ALT = 43

local disabledColor = {
	ZO_DEFAULT_DISABLED_COLOR.r,
	ZO_DEFAULT_DISABLED_COLOR.g,
	ZO_DEFAULT_DISABLED_COLOR.b,
	ZO_DEFAULT_DISABLED_COLOR.a
}
local disabledIcon = {.75,.75,.75,1}

--[[
	large chat bubble = /esoui/art/mainmenu/menubar_notifications_up.dds
	small chat bubble = /esoui/art/chatwindow/chat_notification_up.dds
]]--

local chatterIcons = {
	{
		CHATTER_START_COMPLETE_QUEST,
		CHATTER_ACCEPT_QUEST_BESTOWAL_ALT,
		texture = '/esoui/art/chatwindow/chat_notification_up.dds',
		secondary = {
			placeBehind = true,
			texture = '/esoui/art/chatwindow/chat_notification_over.dds',
			alpha = .9
		}
	},
	{
		CHATTER_COMPLETE_QUEST_CONFIRM_ALT,
		texture = '/esoui/art/buttons/accept_up.dds',
		color = {.5,1,0,1}, anchor = { RIGHT, LEFT, -4, 1 }
	},
	{
		CHATTER_START_NEW_QUEST_BESTOWAL,
		texture = '/esoui/art/inventory/inventory_tabicon_quest_up.dds',
		scale = 1.5, anchor = { RIGHT, LEFT, 2.5, 1 },
		secondary = {
			placeBehind = true,
			texture = '/esoui/art/inventory/inventory_tabicon_quest_over.dds',
			alpha = .7
		}
	},
	{
		CHATTER_START_TALK,
		texture = '/esoui/art/chatwindow/chat_notification_up.dds'
	},
	{
		CHATTER_TALK_CHOICE,
		CHATTER_TALK_CHOICE_PERSUADE_DISABLED,
		CHATTER_TALK_CHOICE_INTIMIDATE_DISABLED,
		texture = '/esoui/art/chatwindow/chat_notification_up.dds'
	},
	{
		CHATTER_GOODBYE,
		texture = '/esoui/art/buttons/decline_up.dds',
		scale = .7, anchor = { RIGHT, LEFT, -9, 1 }
	},
	{
		CHATTER_START_BUY_BAG_SPACE,
		CHATTER_PROMPT_BUY_BAG_SPACE,
		texture = '/esoui/art/mainmenu/menubar_inventory_up.dds',
		anchor = { RIGHT, LEFT, -4, 1 }
	},
	{
		CHATTER_TALK_CHOICE_MONEY,
		texture = '/esoui/art/bank/bank_tabicon_deposit_up.dds'
	},
	{
		CHATTER_START_STABLE,
		texture = '/esoui/art/mounts/tabicon_mounts_up.dds',
		scale = 1.2, anchor = { RIGHT, LEFT, -3, 1 }
	},
	{
		CHATTER_START_SHOP,
		CHATTER_START_TRADINGHOUSE,
		texture = '/esoui/art/guild/guildhistory_indexicon_guildstore_up.dds',
		anchor = { RIGHT, LEFT, -4, 1.5 }
	},
	{
		CHATTER_START_BANK,
		texture = '/esoui/art/guild/guildhistory_indexicon_guildbank_up.dds',
		anchor = { RIGHT, LEFT, -4, 0 }
	},
	{
		CHATTER_START_GUILDBANK,
		texture = '/esoui/art/guild/guildhistory_indexicon_guildbank_up.dds',
		anchor = { RIGHT, LEFT, -4, 0 }
	},
}


local memorizedChatterIcons = {}
function UX_Dialogue.getChatterIcon(optionType)
	if not UX_Dialogue.savedVars.showIcons then return end
	
	local index = memorizedChatterIcons[optionType]
	if index ~= nil then return chatterIcons[index] end

	local l = #chatterIcons
	for i=1,l do
		local item = chatterIcons[i]

		for j,t in ipairs(item) do
			if t == optionType then
				memorizedChatterIcons[optionType] = i
				return item
			end
		end
	end
end


function UX_Dialogue.chatterHandler()
	local numChildren = math.min(9, ZO_InteractWindowPlayerAreaOptions:GetNumChildren())

	for i=1,numChildren do
		local option = ZO_InteractWindowPlayerAreaOptions:GetChild(i)

		local text = option:GetText()

		if #text ~= 0 then
			local modifiedText = text

			-- highlight [Lie], [Persuade] etc.
			if option.enabled and UX_Dialogue.savedVars.highlightSpecial then
				modifiedText = modifiedText:gsub('(%[.-%])', '|cffffff%1|r')
			end

			-- prepend numbers to chat options signifying related keyboard key (1-9)
			if UX_Dialogue.savedVars.showNumbers then
				local prefix = strformat('%d. ', i)
				local from, to = text:find(prefix)

				if from ~= 1 then

					-- highlight shop type
					--[[if option.optionType == CHATTER_START_SHOP then
						modifiedText = modifiedText:gsub('(%(.-%))', '|cffffff%1|r')

					end]]--

					modifiedText = prefix..modifiedText
				end
			end

			option:SetText(modifiedText)
		end

		local icon = _G[option:GetName()..'Icon']
		local texture = _G[option:GetName()..'IconImage']

		-- make sure any previous changes to this icon do not stick
		if option.isTainted then
			icon:SetScale(1)
			
			icon:ClearAnchors()
			icon:SetAnchor(RIGHT, option, LEFT, -5, 0)

			texture:SetColor(1,1,1,1)
			texture:SetDesaturation(0)

			if option.secondary ~= nil then
				option.secondary:SetHidden(true)
				option.secondary:SetAlpha(1)
				option.secondary:SetColor(1,1,1,1)
				option.secondary:SetDesaturation(0)
			end

			option.isTainted = false
		end

		local custom = UX_Dialogue.getChatterIcon(option.optionType)
		if custom ~= nil then

			icon:SetHidden(false)

			if custom.secondary ~= nil then
				if option.secondary == nil then
					option.secondary = UX_DuplicateControl(UX_createName(texture:GetName(), 'Secondary'), texture)
				end

				local secondary = option.secondary

				secondary:SetTexture(custom.secondary.texture)

				secondary:ClearAnchors()
				local point, relativeTo, relativePoint, offsetX, offsetY = select(2, texture:GetAnchor())
				if custom.secondary.offset ~= nil then
					local offsets = custom.secondary.offset
					offsetX = offsetX + offsets[1]
					offsetY = offsetY + offsets[2]
				end
				secondary:SetAnchor(point, relativeTo, relativePoint, offsetX, offsetY)

				secondary:SetScale(custom.secondary.scale or texture:GetScale())

				if custom.secondary.color ~= nil then
					secondary:SetColor(unpack(custom.secondary.color))
				end

				secondary:SetAlpha(custom.secondary.alpha or 1)

				if custom.secondary.placeBehind then
					secondary:SetDrawLevel(texture:GetDrawLevel() - 1)
				else
					secondary:SetDrawLevel(texture:GetDrawLevel() + 1)
				end

				secondary:SetHidden(false)
			end

			texture:SetTexture(custom.texture)

			if custom.scale ~= nil then
				icon:SetScale(custom.scale)
			end

			if custom.anchor ~= nil then
				local point, relativePoint, offsetX, offsetY = unpack(custom.anchor)
				icon:ClearAnchors()
				icon:SetAnchor(point, option, relativePoint, offsetX, offsetY)
			end

			if option.chosenBefore or not option.enabled then
				texture:SetDesaturation(1)
				texture:SetColor(unpack(disabledIcon))

				if option.secondary ~= nil then
					option.secondary:SetDesaturation(1)
					option.secondary:SetColor(unpack(disabledIcon))
				end
			elseif option.isImportant and not custom.colorOverride then
				texture:SetColor(1,0,0,1)

				if option.secondary ~= nil then
					option.secondary:SetColor(1,0,0,1)
				end
			elseif custom.color ~= nil then
				texture:SetColor(unpack(custom.color))
			end

			-- if guild bank or tradehouse and not part of any guild, dim label and icon
			if (option.optionType == CHATTER_START_GUILDBANK or option.optionType == CHATTER_START_TRADINGHOUSE) and GetNumGuilds() == 0 then
				option:SetColor(unpack(disabledColor))
				texture:SetDesaturation(1)
				texture:SetColor(unpack(disabledIcon))
			end

			option.isTainted = true
			texture:SetHidden(false)

		-- flag to avoid resetting unnecessarily
		else
			option.isTainted = false
			icon:SetHidden(true)
		end

	end

	if UX_Dialogue.savedVars.showAudioRepeat and ZO_InteractWindowRewardArea:IsHidden() then
		local text = ZO_InteractWindowTargetAreaBodyText:GetText()

		-- assume non-NPC interaction thus no audio if the body text begins and ends with <> or quotes
		if (text:sub(1,1) == '<' and text:sub(-1) == '>') or (text:sub(1,1) == '"' and text:sub(-1) == '"') then
			UX_Dialogue.audioRepeatControl:SetHidden(true)
		else
			UX_Dialogue.audioRepeatControl:SetHidden(false)
		end
	else
		UX_Dialogue.audioRepeatControl:SetHidden(true)
	end
end


function UX_Dialogue.initAudioRepeat()
	UX_Dialogue.audioRepeatControl = wm:CreateControl(UX_createName('Dialogue', 'AudioRepeat'), ZO_InteractWindowTargetAreaTitle, CT_TEXTURE)
	local audioIcon = UX_Dialogue.audioRepeatControl

	audioIcon:SetAnchor(LEFT, nil, LEFT, -.5, 0)
	audioIcon:SetResizeToFitFile(true)
	audioIcon:SetAlpha(.5)
	audioIcon:SetMouseEnabled(true)

	local textures = {
		up = '/esoui/art/charactercreate/charactercreate_audio_up.dds',
		down = '/esoui/art/charactercreate/charactercreate_audio_down.dds'
	}

	audioIcon:SetTexture(textures.down) -- preload
	audioIcon:SetTexture(textures.up)

	audioIcon:SetHandler('OnMouseEnter', function(self)
		audioIcon:SetAlpha(1)

		if self.isMouseDown then
			self:SetTexture(textures.down)
		else
			self:SetTexture(textures.up)
		end
	end)
	audioIcon:SetHandler('OnMouseExit', function(self)
		self:SetTexture(textures.up)
		audioIcon:SetAlpha(.5)
	end)
	audioIcon:SetHandler('OnMouseDown', function(self, button)
		if wm:GetMouseOverControl() == self and button == MOUSE_BUTTON_LEFT then
			self.isMouseDown = true
			self:SetTexture(textures.down)
		end
	end)

	audioIcon:SetHandler('OnMouseUp', function(self, button)
		self.isMouseDown = false
		self:SetTexture(textures.up)

		-- clicked
		if wm:GetMouseOverControl() == self and button == MOUSE_BUTTON_LEFT then
			-- calling this function also triggers the playing of the dialogue audio
			-- does not work on reward screen
			-- rotates if NPC has multiple messages for the same instance (usually greetings)
			local text, numOptions, atGreeting = GetChatterData()

			-- if the text rotation winds up being different, replace existing text with new
			if ZO_InteractWindowTargetAreaBodyText:GetText() ~= text then
				ZO_InteractWindowTargetAreaBodyText:SetText(text)
			end

			-- This fixes the response options to match the repeated dialogue on quest interactions,
			-- because the audio goes back to the starting stage and the responses were still
			-- reflecting the (possibly later) stage of the conversation where the repeat was initiated.
			-- This is only an issue on quest interactions, because normally the current stage repeats
			-- without having to redo the entire conversation.  -- eventHandler
			if GetInteractionType() == INTERACTION_QUEST then
				ResetChatter()
				UX_Dialogue.chatterHandler()
			end

			-- Support for DelaySubs addon -- eventHandler
			if DELAY_SUBS_LOADED then
				DelaySubs.hideChatter(text)
			end
		end
	end)
end


-- New LibAddonMenu-2.0 options panel replaces original custom panel -- eventHandler
function UX_Dialogue.initOptions()
	local lam = LibStub:GetLibrary("LibAddonMenu-2.0")

	local panelData = {
		type = "panel",
		name = "UX Dialogue",
	}

	lam:RegisterAddonPanel("UX_DIALOGUE_OPTIONS", panelData)

	local optionsData = {
		[1] = {
			type = "checkbox",
			name = "Enable Audio Repeat Button",
			tooltip = 'Repeats or rotates voice-over.\n' .. UX_colorize('Does not work for reward steps.'),
			getFunc = function() return UX_Dialogue.savedVars.showAudioRepeat end,
			setFunc = function() UX_Dialogue.savedVars.showAudioRepeat = not UX_Dialogue.savedVars.showAudioRepeat end,
		},
		[2] = {
			type = "checkbox",
			name = "Number Response Options",
			tooltip = "",
			getFunc = function() return UX_Dialogue.savedVars.showNumbers end,
			setFunc = function() UX_Dialogue.savedVars.showNumbers = not UX_Dialogue.savedVars.showNumbers end,
		},
		[3] = {
			type = "checkbox",
			name = "Display Response Icons",
			tooltip = "",
			getFunc = function() return UX_Dialogue.savedVars.showIcons end,
			setFunc = function() UX_Dialogue.savedVars.showIcons = not UX_Dialogue.savedVars.showIcons end,
		},
		[4] = {
			type = "checkbox",
			name = "Highlight Special Responses",
			tooltip = table.concat({
				'Makes all bracketed options (',
				UX_colorize('[Lie]', 1,1,1),
				', ',
				UX_colorize('[Persuade]', 1,1,1),
				', etc.) stand out.'}),
			getFunc = function() return UX_Dialogue.savedVars.highlightSpecial end,
			setFunc = function() UX_Dialogue.savedVars.highlightSpecial = not UX_Dialogue.savedVars.highlightSpecial end,
		},
	}

	lam:RegisterOptionControls("UX_DIALOGUE_OPTIONS", optionsData)
end


function UX_Dialogue.init(ec, name)
	if name ~= 'UX_Dialogue' then 
		if name == 'DelaySubs' then
			DELAY_SUBS_LOADED = true -- Support for DelaySubs addon - eventHandler
		end
		return
	end

	UX_Dialogue.savedVars = ZO_SavedVars:NewAccountWide('UX_Dialogue_SavedVars', UX_Dialogue.savedVarsVersion, 'Dialogue', UX_Dialogue.defaults)

	UX_Dialogue.initAudioRepeat()

	em:RegisterForEvent(ns, EVENT_CHATTER_BEGIN, UX_Dialogue.chatterHandler)
	em:RegisterForEvent(ns, EVENT_CONVERSATION_UPDATED, UX_Dialogue.chatterHandler)
	em:RegisterForEvent(ns, EVENT_QUEST_OFFERED, UX_Dialogue.chatterHandler)
	em:RegisterForEvent(ns, EVENT_QUEST_COMPLETE_DIALOG, UX_Dialogue.chatterHandler)

	UX_Dialogue.initOptions()
end

em:RegisterForEvent(ns, EVENT_ADD_ON_LOADED, UX_Dialogue.init)
